@@ -67,25 +67,40 @@ clades.from.polytomies <- function(tree) {
   make.clade.tree(tree2, clades.spp)
 }
 
-## All ancestors of node 'x' in 'tree'
-## ancestors <- function(x, tree, tips.only=FALSE) {
-##   from <- tree$edge[,1]
-##   to   <- tree$edge[,2]
-##   n.taxa <- length(tree$tip.label)
-##   is.node <- seq_len(n.taxa) %in% from
-##   anc <- list(x)
-##   n <- 1
-##   while ( length(x) > 0 ) {
-##     kids <- to[from %in% x]
-##     anc[[n <- n + 1]] <- kids
-##     x <- kids[is.node[kids]]
-##   }
-##   anc <- sort(unlist(anc))
-##   if ( tips.only )
-##     anc[anc <= n.taxa]
-##   else
-##     anc
-## }
+## Generate a sample using the algorithm in FitzJohn et al. 2009.
+clades.from.sample <- function(phy, p) {
+  n.taxa <- length(phy$tip.label)
+
+  desc <- lapply(seq_len(phy$Nnode) + n.taxa, get.descendants, phy, TRUE)
+  anc <- diversitree:::ancestors(phy)
+
+  f <- function(i) {
+    check <- rev(na.omit(anc[,i]))[-1] - n.taxa
+    for ( j in check ) {
+      n <- sum(keep[desc[[j]]])
+      if ( n == 1 )
+        return(desc[[j]][keep[desc[[j]]]])
+      else if ( n > 1 )
+        return(NA)
+    }
+  }
+  
+  keep <- runif(n.taxa) < p
+  repeat {
+    drop <- which(!keep)
+    tmp <- sapply(drop, f)
+    if ( any(is.na(tmp)) ) {
+      orphan <- drop[is.na(tmp)]
+      keep[orphan[runif(length(orphan)) < p]] <- TRUE
+    } else {
+      clades <- split(phy$tip.label[drop], phy$tip.label[tmp])
+      break
+    }
+  }
+
+  make.clade.tree(drop.tip.fixed(phy, drop), clades)
+}
+
 
 ## Renamed poorly because of a clash with util.R:ancestors
 ## TODO: I believe that this is actually descendents()
