@@ -12,8 +12,8 @@ make.mkn.multitrait <- function(tree, states, depth=NULL,
   cache <- make.cache.mkn.multitrait(tree, states, depth,
                                      allow.multistep, strict,
                                      control)
-  lik.mkn <- make.mkn(tree, cache$states.mkn, cache$info$k, FALSE,
-                      cache$control.mkn)
+  lik.mkn <- make.mkn(tree, cache$states.mkn, cache$info$k,
+                      strict=FALSE, control=cache$control.mkn)
   f.pars <- make.pars.mkn.multitrait(cache)
 
   ll <- function(pars, root=ROOT.OBS, root.p=NULL,
@@ -46,7 +46,7 @@ make.cache.mkn.multitrait <- function(tree, states, depth,
   k <- 2^n.trait
   
   if ( is.null(control$method) )
-    control$method <- if (n.trait > 3) "ode" else "mkn"
+    control$method <- if (n.trait > 3) "ode" else "exp"
 
   states <- check.states.musse.multitrait(tree, states, strict=strict,
                                           strict.vals=0:1)
@@ -55,6 +55,8 @@ make.cache.mkn.multitrait <- function(tree, states, depth,
   if ( any(is.na(states)) )
     stop("Missing data not yet allowed")
 
+  cache <- list()
+
   ## #### Make new states
   ## TODO/NEW:
   ## I think that this is actually somewhat general, and something
@@ -72,7 +74,7 @@ make.cache.mkn.multitrait <- function(tree, states, depth,
   key <- apply(do.call(expand.grid, rep(list(0:1), n.trait)),
                1, paste, collapse="")
 
-  states.mkn <- match(code, types)
+  states.mkn <- match(code, key)
   names(states.mkn) <- rownames(states)
   cache$states.mkn <- states.mkn
   ## #### End
